import logging
import requests
import json

from .base import Provider as BaseProvider

LOGGER = logging.getLogger(__name__)

def ProviderParser(subparser):
    subparser.add_argument('--auth-key', help='specify the key to access the API')
    subparser.add_argument('--auth-secret', help='specify the secret to access the API')

class Provider(BaseProvider):

    def __init__(self, options, engine_overrides=None):
        super(Provider, self).__init__(options, engine_overrides)
        self.domain_id = None
        self.api_endpoint = self.engine_overrides.get('api_endpoint', 'https://api.godaddy.com/v1')

    def authenticate(self):
        domain = self.options.get('domain')

        result = self._get('/domains/{0}'.format(domain))
        self.domain_id = result['domainId']

    def create_record(self, type, name, content):
        domain = self.options.get('domain')
        ttl = self.options.get('ttl')

        data = {'data': content}
        if ttl:
            data['ttl'] = ttl

        self._put('/domains/{0}/records/{1}/{2}'
                  .format(domain, type, self._relative_name(name)), [data])

        LOGGER.debug('create_record: %s %s %s', type, name, content)

        return True

    def list_records(self, type=None, name=None, content=None):
        domain = self.options.get('domain')
        records = []

        url = '/domains/{0}/records'.format(domain)
        if type:
            url += '/{0}'.format(type)
        if name:
            url += '/{0}'.format(self._relative_name(name))

        raws = self._get(url)

        for raw in raws:
            records.append({
                'type': raw['type'],
                'name': self._full_name(raw['name']),
                'ttl': raw['ttl'],
                'content': raw['data']
            })

        if content:
            records = [record for record in records if record['data'].lower() == content.lower()]

        LOGGER.debug('list_records: %s', records)

        return records

    def update_record(self, identifier, type=None, name=None, content=None):
        # With GoDaddy API, creating a record for given type and name is the same
        #   than updating the record.
        return self.create_record(type, name, content)

    def delete_record(self, identifier=None, type=None, name=None, content=None):
        domain = self.options.get('domain')

        if not type:
            raise Exception('ERROR: type is required')
        if not name:
            raise Exception('ERROR: name is required')
        if not content:
            raise Exception('ERROR: content is required')

        # OK some explanations need to be done here.
        # GoDaddy DNS API does not provide a direct way to delete a record (weird).
        # However it provides a way to get and update all records of a zone.
        # So :
        #  - we get all the records,
        #  - we filter the array to remove the record to be deleted,
        #  - then we push back the filtered array to set the zone without the record to be deleted.
        # And yes, we could limit the operation to a given type record (eg. TXT, there is an URL
        #   for that), but GoDaddy refuses to push back an empty set of a given type (yep, you
        #   cannot remove all your TXT with this URL, ultra weird).
        # It is likely to happen during a DNS challenge, as all TXT should be removed at the end.
        # So operating on all the zone avoid empty sets (there will always at least NS entries).
        records = self._get('/domains/{0}/records'.format(domain))
        to_insert = [record for record in records
                     if record['type'].lower() != type.lower()
                     or record['name'].lower() != self._relative_name(name).lower()
                     or record['data'].lower() != content.lower()]

        num_to_delete = len(records) - len(to_insert)

        if num_to_delete > 1:
            raise Exception('ERROR: multiple records marked to be deleted')

        self._put('/domains/{0}/records'.format(domain), to_insert)

        LOGGER.debug('delete_record: %s', num_to_delete != 0)

        return num_to_delete != 0

    def _request(self, action='GET', url='/', data=None, query_params=None):
        if not data:
            data = {}
        if not query_params:
            query_params = {}

        result = requests.request(action, self.api_endpoint + url,
                                  params=query_params,
                                  data=json.dumps(data),
                                  headers={
                                      'Content-Type': 'application/json',
                                      'Accept': 'application/json',
                                      # GoDaddy use a key/secret pair to authenticate
                                      'Authorization': 'sso-key {0}:{1}'.format(
                                          self.options.get('auth_key'),
                                          self.options.get('auth_secret'))
                                  })

        result.raise_for_status()
        return result.json()



from __future__ import absolute_import
from __future__ import print_function

import logging


from .base import Provider as BaseProvider

try:
    import namecheap #optional dep
except ImportError:
    pass

logger = logging.getLogger(__name__)


def ProviderParser(subparser):
    subparser.add_argument(
        '--auth-token',
        help='specify api token used to authenticate'
    )
    subparser.add_argument(
        '--auth-username',
        help='specify email address used to authenticate'
    )
    # FIXME What is the client IP used for?
    subparser.add_argument(
        '--auth-client-ip',
        help='Client IP address to send to Namecheap API calls',
        default='127.0.0.1'
    )
    subparser.add_argument(
        '--auth-sandbox',
        help='Whether to use the sandbox server',
        action='store_true'
    )

class Provider(BaseProvider):

    def __init__(self, options, engine_overrides=None):
        super(Provider, self).__init__(options, engine_overrides)
        self.options = options
        self.client = namecheap.Api(
            ApiUser=options.get('auth_username',''),
            ApiKey=options.get('auth_token',''),
            UserName=options.get('auth_username',''),
            ClientIP=options.get('auth_client_ip',''),
            sandbox=options.get('auth_sandbox', False),
            debug=False
        )
        self.domain = self.options['domain']
        self.domain_id = None

    def authenticate(self):
        try:
            domain_names = [x['Name'] for x in self.client.domains_getList()]
        except namecheap.ApiError:
            raise Exception('Authentication failed')
        if self.domain not in domain_names:
            raise Exception('The domain {} is not controlled by this Namecheap '
                            'account'.format(self.domain))
        # FIXME What is this for?
        self.domain_id = self.domain

    # Create record. If record already exists with the same content, do nothing
    def create_record(self, type, name, content):
        record = {
            # required
            'Type': type,
            'Name': self._relative_name(name),
            'Address': content
        }
        # logger.debug('create_record: %s', 'id' in payload)
        # return 'id' in payload
        self.client.domains_dns_addHost(self.domain, record)
        return True

    # List all records. Return an empty list if no records found.
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is
    # received.
    def list_records(self, type=None, name=None, content=None, id=None):


        records = []
        raw_records = self.client.domains_dns_getHosts(self.domain)
        for record in raw_records:
            records.append(self._convert_to_lexicon(record))

        if id:
            records = [record for record in records if record['id'] == id]
        if type:
            records = [record for record in records if record['type'] == type]
        if name:
            if name.endswith('.'):
                name = name[:-1]
            records = [record for record in records if name in record['name'] ]
        if content:
            records = [record for record in records if record['content'].lower() == content.lower()]

        logger.debug('list_records: %s', records)
        return records

    # Create or update a record.
    def update_record(self, identifier, type=None, name=None, content=None):
        # Delete record if it exists
        self.delete_record(identifier, type, name, content)
        return self.create_record(type, name, content)

    # Delete an existing record.
    # If record does not exist, do nothing.
    def delete_record(self, identifier=None, type=None, name=None, content=None):
        records = self.list_records(type=type, name=name, content=content, id=identifier)
        for record in records:
            self.client.domains_dns_delHost(self.domain, self._convert_to_namecheap(record))
        
        return True

    def _convert_to_namecheap(self, record):
        """ converts from lexicon format record to namecheap format record,
        suitable to sending through the api to namecheap"""

        name = record['name']
        if name.endswith('.'):
            name = name[:-1]

        short_name = name[:name.find(self.domain)-1]
        processed_record = {
            'Type': record['type'],
            'Name': short_name,
            'TTL': record['ttl'],
            'Address': record['content'],
            'HostId': record['id']
        }

        return processed_record

    def _convert_to_lexicon(self, record):
        """ converts from namecheap raw record format to lexicon format record
        """

        name = record['Name']
        if self.domain not in name:
            name = "{}.{}".format(name,self.domain)

        processed_record = {
            'type': record['Type'],
            'name': '{0}.{1}'.format(record['Name'], self.domain),
            'ttl': record['TTL'],
            'content': record['Address'],
            'id': record['HostId']
        }

        return processed_record



from __future__ import absolute_import
from __future__ import print_function

import logging
import requests

from .base import Provider as BaseProvider

logger = logging.getLogger(__name__)


def ProviderParser(subparser):
    identity_group = subparser.add_mutually_exclusive_group()
    identity_group.add_argument("--auth-id", help="specify user id used to authenticate")
    identity_group.add_argument("--auth-subid", help="specify subuser id used to authenticate")
    identity_group.add_argument("--auth-subuser", help="specify subuser name used to authenticate")
    subparser.add_argument("--auth-password", help="specify password used to authenticate")
    subparser.add_argument("--weight", help="specify the SRV record weight")
    subparser.add_argument("--port", help="specify the SRV record port")


class Provider(BaseProvider):
    def __init__(self, options, engine_overrides=None):
        super(Provider, self).__init__(options, engine_overrides)
        self.domain_id = None
        self.api_endpoint = self.engine_overrides.get('api_endpoint', 'https://api.cloudns.net')

    def authenticate(self):
        payload = self._get('/dns/get-zone-info.json', {'domain-name': self.options['domain']})
        self.domain_id = payload['name']
        logger.debug('authenticate: %s', payload)

    def create_record(self, type, name, content):
        # Skip execution if such a record already exists
        existing_records = self.list_records(type, name, content)
        if len(existing_records) > 0:
            return True

        # Build parameters for adding a new record
        params = {
            'domain-name': self.domain_id,
            'record-type': type,
            'host': self._relative_name(name),
            'record': content
        }
        if self.options['ttl']:
            params['ttl'] = self.options['ttl']
        if self.options['priority']:
            params['priority'] = self.options['priority']
        if self.options['weight']:
            params['weight'] = self.options['weight']
        if self.options['port']:
            params['port'] = self.options['port']

        # Add new record by calling the ClouDNS API
        payload = self._post('/dns/add-record.json', params)
        logger.debug('create_record: %s', payload)

        # Error handling is already covered by self._request
        return True

    def list_records(self, type=None, name=None, content=None):
        # Build parameters to make use of the built-in API filtering
        params = {'domain-name': self.domain_id}
        if type:
            params['type'] = type
        if name:
            params['host'] = self._relative_name(name)

        # Fetch and parse all records for the given zone
        payload = self._get('/dns/records.json', params)
        payload = payload if not isinstance(payload, list) else {}
        records = []
        for record in payload.values():
            records.append({
                'type': record['type'],
                'name': self._full_name(record['host']),
                'ttl': record['ttl'],
                'content': record['record'],
                'id': record['id']
            })

        # Filter by content manually as API does not support that
        if content:
            records = [record for record in records if record['content'] == content]

        # Print records as debug output and return them
        logger.debug('list_records: %s', records)
        return records

    def update_record(self, identifier, type=None, name=None, content=None):
        # Try to find record if no identifier was specified
        if not identifier:
            identifier = self._find_record_identifier(type, name, None)

        # Build parameters for updating an existing record
        params = {'domain-name': self.domain_id, 'record-id': identifier}
        if name:
            params['host'] = self._relative_name(name)
        if content:
            params['record'] = content
        if self.options.get('ttl'):
            params['ttl'] = self.options.get('ttl')
        if self.options['priority']:
            params['priority'] = self.options['priority']
        if self.options['weight']:
            params['weight'] = self.options['weight']
        if self.options['port']:
            params['port'] = self.options['port']

        # Update existing record by calling the ClouDNS API
        payload = self._post('/dns/mod-record.json', params)
        logger.debug('update_record: %s', payload)

        # Error handling is already covered by self._request
        return True

    def delete_record(self, identifier=None, type=None, name=None, content=None):
        # Try to find record if no identifier was specified
        delete_record_id = []
        if not identifier:
            records = self.list_records(type, name, content)
            delete_record_id = [record['id'] for record in records]
        else:
            delete_record_id.append(identifier)
        
        logger.debug('delete_records: %s', delete_record_id)
        
        for record_id in delete_record_id:
            # Delete existing record by calling the ClouDNS API
            payload = self._post('/dns/delete-record.json', {'domain-name': self.domain_id, 'record-id': record_id})
        
        logger.debug('delete_record: %s', True)

        # Error handling is already covered by self._request
        return True

    def _build_authentication_data(self):
        if not self.options['auth_password']:
            raise Exception('No valid authentication data passed, expected: auth-password')

        if self.options['auth_id']:
            return {'auth-id': self.options['auth_id'], 'auth-password': self.options['auth_password']}
        elif self.options['auth_subid']:
            return {'sub-auth-id': self.options['auth_subid'], 'auth-password': self.options['auth_password']}
        elif self.options['auth_subuser']:
            return {'sub-auth-user': self.options['auth_subuser'], 'auth-password': self.options['auth_password']}
        else:
            raise Exception('No valid authentication data passed, expected: auth-id, auth-subid, auth-subuser')

    def _find_record_identifier(self, type, name, content):
        records = self.list_records(type, name, content)
        logger.debug('records: %s', records)
        if len(records) == 1:
            return records[0]['id']
        else:
            raise Exception('Record identifier could not be found.')

    def _request(self, action='GET', url='/', data=None, query_params=None):
        # Set default values for missing arguments
        data = data if data else {}
        query_params = query_params if query_params else {}

        # Merge authentication data into request
        if action == 'GET':
            query_params.update(self._build_authentication_data())
        else:
            data.update(self._build_authentication_data())

        # Fire request against ClouDNS API and parse result as JSON
        r = requests.request(action, self.api_endpoint + url, params=query_params, data=data)
        r.raise_for_status()
        payload = r.json()

        # Check ClouDNS specific status code and description
        if 'status' in payload and 'statusDescription' in payload and payload['status'] != 'Success':
            raise Exception('ClouDNS API request has failed: ' + payload['statusDescription'])

        # Return payload
        return payload
